/**************************************************************
 SVPFlow1 - motion vectors search for SmoothVideo Project (SVP)
 Based on MVTools by Manao
 Copyright(c)2011-2016 Chainik - https://www.svp-team.com
 Copyright(c)2006 A.G.Balakhnin aka Fizick - http://avisynth.org.ru

 Avisynth: http://www.avisynth.nl
 Vapoursynth: http://www.vapoursynth.com
 JsonCpp: jsoncpp.sourceforge.net under MIT

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA, or visit
 http://www.gnu.org/copyleft/gpl.html
*****************************************************************/

#include "common.h"
#include "version.h"
#include "mvanalyse.h"
#include "mvsuper.h"

extern "C"
#ifdef Q_OS_WIN
__declspec(dllexport) 
#endif
uint32_t svpGetVersion()
{
	return ((SVP_PLUGIN_VERSION << 16) | VCS_REVISION);
}

#ifdef AVISYNTH_PLUGIN

#include "vectorsconverter.h"

AVSValue __cdecl Create_SVSuper(AVSValue args, void*, IScriptEnvironment* env)
{
    JsonSVP::Value root;
    JsonSVP::Reader reader;
    if(!reader.parse(args[1].AsString("{}"),root))
        env->ThrowError(("SVSuper: invalid 'params' syntax: "+reader.getFormatedErrorMessages()).c_str());

    return new MVSuper_AVS(
                args[0].AsClip(), // source
                root, env);
}

AVSValue __cdecl Create_SVAnalyse(AVSValue args, void*, IScriptEnvironment* env)
{
    JsonSVP::Value root;
    JsonSVP::Reader reader;
    if(!reader.parse(args[1].AsString("{}"),root))
        env->ThrowError(("SVAnalyse: invalid 'params' syntax: "+reader.getFormatedErrorMessages()).c_str());

    return new MVAnalyse_AVS(
                args[0].AsClip(), // super
                (args[2].Defined() ? args[2].AsClip() : 0), //source
                root,env);
}

AVSValue __cdecl Create_SVConvert(AVSValue args, void*, IScriptEnvironment* env)
{
    return new VectorsConverter(args[0].AsClip(),args[1].AsBool(false),env);
}

const AVS_Linkage *AVS_linkage = 0;

extern "C" __declspec(dllexport)
const char* __stdcall AvisynthPluginInit3(IScriptEnvironment* env, const AVS_Linkage* const vectors)
{
	AVS_linkage = vectors;

    env->AddFunction("SVSuper", "cs", Create_SVSuper, 0);
    env->AddFunction("SVAnalyse", "cs[src]c", Create_SVAnalyse, 0);
    env->AddFunction("SVConvert", "c[isb]b", Create_SVConvert, 0);

	return("svpflow1: part of SmoothVideo Project frame interpolation engine based on MVTools2 plugin");
}

#endif //AVISYNTH_PLUGIN

#ifdef VAPOURSYNTH_PLUGIN

VS_EXTERNAL_API(void) VapourSynthPluginInit(VSConfigPlugin configFunc, VSRegisterFunction registerFunc, VSPlugin *plugin)
{
	configFunc("com.svp-team.flow1", "svp1", "SVPFlow1", VAPOURSYNTH_API_VERSION, 1, plugin);

	registerFunc("Super", "clip:clip;opt:data", &MVSuper_VS::create, 0, plugin);
	registerFunc("Analyse", "clip:clip;sdata:int;src:clip;opt:data", &MVAnalyse_VS::create, 0, plugin);
}

#endif
