/**************************************************************
 SVPFlow1 - motion vectors search for SmoothVideo Project (SVP)
 Based on MVTools by Manao
 Copyright(c)2011 Chainik - http://www.svp-team.com
 Copyright(c)2006 A.G.Balakhnin aka Fizick - http://avisynth.org.ru

 JsonCpp: jsoncpp.sourceforge.net under MIT

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA, or visit
 http://www.gnu.org/copyleft/gpl.html
*****************************************************************/

#pragma once

#include <json/json.h>

#include "basefilter.h"

#include "mvframe.h"
#include "groupofplanes.h"
#include "mvsuper.h"

class MVAnalyseCore:public CoreLogic
{
public:
    MVAnalyseCore(const JsonSVP::Value& root, BaseFilter *base);
    ~MVAnalyseCore();

    void initVideoInfo(BaseFilter::Video *vi);

    void prepare(int n, void *baseRef);
    std::shared_ptr<BaseFilter::Frame> get(int n, void *baseRef);

protected:
    void initVectorFields(std::vector<GroupOfPlanes*>& vf);
    void checkRemoteMessages();

protected:
    SVAnalysisData ad;
    MVSuperCore::Params superParams;

    SearchType searchType;
    int searchParam; // search radius at finest level
    int plen; // penalty factor (similar to lambda) for vector length - added by Fizick
    double plevel; // penalty factors (lambda, plen) level scaling - added by Fizick
    int pglobal; // penalty factor for global motion predictor
    int pzero; // penalty factor for zero vector
    int pnbour;  // penalty factor for neighbour vectors
    int preverse;
    int badrange;// range (radius) of wide search
    bool tryMany; // try refine around many predictors
    bool sorting;

    int headerSize;
    int totalVectorsCount;

    bool hasFinestLevel;
    std::vector<GroupOfPlanes::InitData> vfData;
};

#ifdef AVISYNTH_PLUGIN

class MVAnalyse_AVS:public GenericVideoFilter
{
public:
    MVAnalyse_AVS(PClip _child, PClip source, const JsonSVP::Value& root, IScriptEnvironment* env);
    ~MVAnalyse_AVS();

    PVideoFrame __stdcall GetFrame(int n, IScriptEnvironment* env);

protected:
    uint64_t handle;
    std::shared_ptr<MVAnalyseCore> core;
};

#endif

#ifdef VAPOURSYNTH_PLUGIN
class MVAnalyse_VS
{
public:
    static void VS_CC create(const VSMap *in, VSMap *out, void *userData, VSCore *core, const VSAPI *vsapi);

    static void VS_CC init(VSMap *in, VSMap *out, void **instanceData, VSNode *node, VSCore *core, const VSAPI *vsapi);
    static const VSFrameRef* VS_CC getFrame(int n, int activationReason, void **instanceData, void **frameData, VSFrameContext *frameCtx, VSCore *core, const VSAPI *vsapi);
    static void VS_CC free(void *instanceData, VSCore *core, const VSAPI *vsapi);
};
#endif
