/**************************************************************
 SVPFlow1 - motion vectors search for SmoothVideo Project (SVP)
 Based on MVTools by Manao
 Copyright(c)2011-2016 Chainik - http://www.svp-team.com
 Copyright(c)2006 A.G.Balakhnin aka Fizick - http://avisynth.org.ru

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA, or visit
 http://www.gnu.org/copyleft/gpl.html
*****************************************************************/

#pragma once

#include <json/json.h>

#include "basefilter.h"
#include "mvframe.h"

#define SUPER_REMOTECONTROL 1
#define SUPER_NOFINESTLEVEL 2

class MVSuperCore:public CoreLogic
{
public:
    // MVSuper parameters packed to 64 bit
    struct Params
    {
        unsigned short nHeight;
        unsigned short nWidth;
        uint8_t nPel;
        uint8_t nLevels;
        uint8_t gpuMode;
        uint8_t flags; //see SUPER_xxx
    };

public:
    MVSuperCore(const JsonSVP::Value& root,BaseFilter *base);
    ~MVSuperCore();

    void initVideoInfo(BaseFilter::Video *vi);

    void prepare(int n, void *baseRef);
    std::shared_ptr<BaseFilter::Frame> get(int n, void *baseRef);

    static unsigned int planeSuperOffset(bool chroma, int src_height, int level, int pel, int plane_pitch, bool hasFinestLevel);
    static int planeSize(int src_sz, int level);

protected:
    void checkRemoteMessages();

protected:
    Params params;

    int sharp;
    int rfilter; // frame reduce filter mode
    int gpuMode;
    bool needFinestLevel;
};

#ifdef AVISYNTH_PLUGIN

class MVSuper_AVS:public GenericVideoFilter
{
public:
    MVSuper_AVS(PClip _child, const JsonSVP::Value& root, IScriptEnvironment* env);
    ~MVSuper_AVS();

    PVideoFrame __stdcall GetFrame(int n, IScriptEnvironment* env);

protected:
    uint64_t handle;
    std::shared_ptr<MVSuperCore> core;
};

#endif

#ifdef VAPOURSYNTH_PLUGIN
class MVSuper_VS
{
public:
    static void VS_CC create(const VSMap *in, VSMap *out, void *userData, VSCore *core, const VSAPI *vsapi);

    static void VS_CC init(VSMap *in, VSMap *out, void **instanceData, VSNode *node, VSCore *core, const VSAPI *vsapi);
    static const VSFrameRef* VS_CC getFrame(int n, int activationReason, void **instanceData, void **frameData, VSFrameContext *frameCtx, VSCore *core, const VSAPI *vsapi);
    static void VS_CC free(void *instanceData, VSCore *core, const VSAPI *vsapi);
};
#endif
